// UBPile Admin Page -- Activity Log

Class.add( Page.Admin, {
	
	activity_types: {
		'^cat': '<i class="fa fa-folder-open-o">&nbsp;</i>Категория',
		'^group': '<i class="mdi mdi-server-network">&nbsp;</i>Группа',
		'^plugin': '<i class="fa fa-plug">&nbsp;</i>Плагин',
		// '^apikey': '<i class="fa fa-key">&nbsp;</i>API-ключ',	
		'^apikey': '<i class="mdi mdi-key-variant">&nbsp;</i>API-ключ',	
		'^event': '<i class="fa fa-clock-o">&nbsp;</i>Событие',
		'^user': '<i class="fa fa-user">&nbsp;&nbsp;</i>Пользователь',
		'server': '<i class="mdi mdi-desktop-tower mdi-lg">&nbsp;</i>Сервер',
		'^job': '<i class="fa fa-pie-chart">&nbsp;</i>Задание',
		'^state': '<i class="mdi mdi-calendar-clock">&nbsp;</i>Планировщик', // mdi-lg
		'^error': '<i class="fa fa-exclamation-triangle">&nbsp;</i>Ошибка',
		'^warning': '<i class="fa fa-exclamation-circle">&nbsp;</i>Предупреждение'
	},
	
	gosub_activity: function(args) {
		// show activity log
		app.setWindowTitle( "Журнал активности" );
		
		if (!args.offset) args.offset = 0;
		if (!args.limit) args.limit = 25;
		app.api.post( 'app/get_activity', copy_object(args), this.receive_activity.bind(this) );
	},
	
	receive_activity: function(resp) {
		// receive page of activity from server, render it
		this.lastActivityResp = resp;
		
		var html = '';
		this.div.removeClass('loading');
		
		html += this.getSidebarTabs( 'activity',
			[
				['activity', "Журнал активности"],
				['api_keys', "API-ключи"],
				['categories', "Категории"],
				['plugins', "Плагины"],
				['servers', "Серверы"],
				['users', "Пользователи"]
			]
		);
		
		this.events = [];
		if (resp.rows) this.events = resp.rows;
		
		var cols = ['Дата/Время', 'Тип', 'Описание', 'Имя пользователя', 'IP адрес', 'Действия'];
		
		html += '<div style="padding:20px 20px 30px 20px">';
		
		html += '<div class="subtitle">';
			html += 'Журнал активности';
			// html += '<div class="clear"></div>';
		html += '</div>';
		
		var self = this;
		html += this.getPaginatedTable( resp, cols, 'объектов', function(item, idx) {
			// figure out icon first
			if (!item.action) item.action = 'unknown';
			
			var item_type = '';
			for (var key in self.activity_types) {
				var regexp = new RegExp(key);
				if (item.action.match(regexp)) {
					item_type = self.activity_types[key];
					break;
				}
			}
			
			// compose nice description
			var desc = '';
			var actions = [];
			var color = '';
			
			switch (item.action) {
				
				// categories
				case 'cat_create':
					desc = 'Создана новая категория: <b>' + item.cat.title + '</b>';
				break;
				case 'cat_update':
					desc = 'Категория обновлена: <b>' + item.cat.title + '</b>';
				break;
				case 'cat_delete':
					desc = 'Категория удалена: <b>' + item.cat.title + '</b>';
				break;
				
				// groups
				case 'group_create':
					desc = 'Создана новая группа серверов: <b>' + item.group.title + '</b>';
				break;
				case 'group_update':
					desc = 'Группа серверов обновлена: <b>' + item.group.title + '</b>';
				break;
				case 'group_delete':
					desc = 'Группа серверов удалена: <b>' + item.group.title + '</b>';
				break;
				
				// plugins
				case 'plugin_create':
					desc = 'Группа серверов удалена: <b>' + item.plugin.title + '</b>';
				break;
				case 'plugin_update':
					desc = 'Создан новый плагин: <b>' + item.plugin.title + '</b>';
				break;
				case 'plugin_delete':
					desc = 'Плагин удален: <b>' + item.plugin.title + '</b>';
				break;
				
				// api keys
				case 'apikey_create':
					desc = 'Создан новый ключ API: <b>' + item.api_key.title + '</b> (Ключ: ' + item.api_key.key + ')';
					actions.push( '<a href="#Admin?sub=edit_api_key&id='+item.api_key.id+'">Изменить ключ</a>' );
				break;
				case 'apikey_update':
					desc = 'Ключ API обновлен: <b>' + item.api_key.title + '</b> (Ключ: ' + item.api_key.key + ')';
					actions.push( '<a href="#Admin?sub=edit_api_key&id='+item.api_key.id+'">Изменить ключ</a>' );
				break;
				case 'apikey_delete':
					desc = 'Ключ API удален: <b>' + item.api_key.title + '</b> (Ключ: ' + item.api_key.key + ')';
				break;
				
				// events
				case 'event_create':
					desc = 'Добавлено новое событие: <b>' + item.event.title + '</b>';
					desc += " (" + summarize_event_timing(item.event.timing, item.event.timezone) + ")";
					actions.push( '<a href="#Schedule?sub=edit_event&id='+item.event.id+'">Изменить событие</a>' );
				break;
				case 'event_update':
					desc = 'Событие обновлено: <b>' + item.event.title + '</b>';
					actions.push( '<a href="#Schedule?sub=edit_event&id='+item.event.id+'">Изменить событие</a>' );
				break;
				case 'event_delete':
					desc = 'Событие удалено: <b>' + item.event.title + '</b>';
				break;
				
				// users
				case 'user_create':
					desc = 'Создана новая учетная запись пользователя: <b>' + item.user.username + "</b> (" + item.user.full_name + ")";
					actions.push( '<a href="#Admin?sub=edit_user&username='+item.user.username+'">Изменить пользователя</a>' );
				break;
				case 'user_update':
					desc = 'Аккаунт пользователя обновлен: <b>' + item.user.username + "</b> (" + item.user.full_name + ")";
					actions.push( '<a href="#Admin?sub=edit_user&username='+item.user.username+'">Изменить пользователя</a>' );
				break;
				case 'user_delete':
					desc = 'Учетная запись пользователя удалена: <b>' + item.user.username + "</b> (" + item.user.full_name + ")";
				break;
				case 'user_login':
					desc = "Пользователь вошел в систему: <b>" + item.user.username + "</b> (" + item.user.full_name + ")";
				break;
				
				// servers
				case 'add_server': // legacy
				case 'server_add': // current
					desc = 'Сервер '+(item.manual ? 'вручную ' : '')+'добавлено в кластер: <b>' + item.hostname + '</b>';
				break;
				case 'remove_server': // legacy
				case 'server_remove': // current
					desc = 'Сервер '+(item.manual ? 'вручную ' : '')+'удалено из кластера: <b>' + item.hostname + '</b>';
				break;
				case 'master_server': // legacy
				case 'server_master': // current
					desc = 'Сервер стал основным: <b>' + item.hostname + '</b>';
				break;
				
				case 'server_restart': 
					desc = 'Сервер перезапущен: <b>' + item.hostname + '</b>';
				break;
				case 'server_shutdown': 
					desc = 'Сервер выключен: <b>' + item.hostname + '</b>';
				break;
				
				case 'server_disable': 
					desc = 'Потеряно соединение с сервером: <b>' + item.hostname + '</b>';
					color = 'yellow';
				break;
				case 'server_enable': 
					desc = 'Повторное подключение к серверу: <b>' + item.hostname + '</b>';
				break;
				
				// jobs
				case 'job_run':
					var event = find_object( app.schedule, { id: item.event } ) || { title: 'Неизвестное событие' };
					desc = 'Задание <b>#'+item.id+'</b> ('+event.title+') запускается вручную';
					actions.push( '<a href="#JobDetails?id='+item.id+'">Подробности о работе</a>' );
				break;
				case 'job_complete':
					var event = find_object( app.schedule, { id: item.event } ) || { title: 'Неизвестное событие' };
					if (!item.code) {
						desc = 'Задание <b>#'+item.id+'</b> ('+event.title+') на сервере <b>'+item.hostname.replace(/\.[\w\-]+\.\w+$/, '')+'</b> завершено успешно';
					}
					else {
						desc = 'Задание <b>#'+item.id+'</b> ('+event.title+') на сервере <b>'+item.hostname.replace(/\.[\w\-]+\.\w+$/, '')+'</b> произошел сбой из-за ошибки: ' + encode_entities(item.description || 'Неизвестная ошибка');
						if (desc.match(/\n/)) desc = desc.split(/\n/).shift() + "...";
						color = 'red';
					}
					actions.push( '<a href="#JobDetails?id='+item.id+'">Подробности о работе</a>' );
				break;
				case 'job_delete':
					var event = find_object( app.schedule, { id: item.event } ) || { title: 'Неизвестное событие' };
					desc = 'Задание <b>#'+item.id+'</b> ('+event.title+') удалено вручную';
				break;
				
				// scheduler
				case 'state_update':
					desc = 'Планировщик был <b>' + (item.enabled ? 'включено' : 'выключено') + '</b>';
				break;
				
				// errors
				case 'error':
					desc = encode_entities( item.description );
					color = 'red';
				break;
				
				// warnings
				case 'warning':
					desc = encode_entities( item.description );
					color = 'yellow';
				break;
				
			} // action
			
			var tds = [
				'<div style="white-space:nowrap;">' + (new Date(item.epoch * 1000 || 0)).toLocaleDateString() + '</div>',
				'<div class="td_big" style="white-space:nowrap; font-size:12px; font-weight:normal;">' + item_type + '</div>',
				'<div class="activity_desc">' + desc + '</div>',
				'<div style="white-space:nowrap;">' + self.getNiceUsername(item, true) + '</div>',
				(item.ip || 'н/д').replace(/^\:\:ffff\:(\d+\.\d+\.\d+\.\d+)$/, '$1'),
				'<div style="white-space:nowrap;">' + actions.join(' | ') + '</div>'
			];
			if (color) tds.className = color;
			
			return tds;
		} );
		
		html += '</div>'; // padding
		html += '</div>'; // sidebar tabs
		
		this.div.html( html );
	}
	
});