// UBPile Admin Page -- Servers

Class.add( Page.Admin, {
	
	gosub_servers: function(args) {
		// show server list, server groups
		this.div.removeClass('loading');
		app.setWindowTitle( "Серверы" );
		
		var size = get_inner_window_size();
		var col_width = Math.floor( ((size.width * 0.9) + 400) / 9 );
		
		var html = '';
		
		html += this.getSidebarTabs( 'servers',
			[
				['activity', "Журнал активности"],
				['api_keys', "API-ключи"],
				['categories', "Категории"],
				['plugins', "Плагины"],
				['servers', "Серверы"],
				['users', "Пользователи"]
			]
		);
		
		html += '<div style="padding:20px 20px 30px 20px">';
		
		// Active Server Cluster
		
		var cols = ['Имя хоста', 'IP адрес', 'Группы', 'Статус', 'Активные задания', 'Время работы', 'ЦП', 'ОЗУ', 'Действия'];
		
		html += '<div class="subtitle">';
			html += 'Кластер серверов';
			// html += '<div class="clear"></div>';
		html += '</div>';
		
		this.servers = [];
		var hostnames = hash_keys_to_array( app.servers ).sort();
		for (var idx = 0, len = hostnames.length; idx < len; idx++) {
			this.servers.push( app.servers[ hostnames[idx] ] );
		}
		
		// include nearby servers under main server list
		if (app.nearby) {
			var hostnames = hash_keys_to_array( app.nearby ).sort();
			for (var idx = 0, len = hostnames.length; idx < len; idx++) {
				var server = app.nearby[ hostnames[idx] ];
				if (!app.servers[server.hostname]) {
					server.nearby = 1;
					this.servers.push( server );
				}
			}
		}
		
		// render table
		var self = this;
		html += this.getBasicTable( this.servers, cols, 'серверов', function(server, idx) {
			
			// render nearby servers differently
			if (server.nearby) {
				var tds = [
					'<div class="td_big" style="font-weight:normal"><div class="ellip" style="max-width:'+col_width+'px;"><i class="fa fa-eye">&nbsp;</i>' + server.hostname.replace(/\.[\w\-]+\.\w+$/, '') + '</div></div>',
					(server.ip || 'н/д').replace(/^\:\:ffff\:(\d+\.\d+\.\d+\.\d+)$/, '$1'),
					'-', '(Рядом)', '-', '-', '-', '-',
					'<span class="link" onMouseUp="$P().add_server_from_list('+idx+')"><b>Добавить</b></span>'
				];
				tds.className = 'blue';
				return tds;
			} // nearby
			
			var actions = [
				'<span class="link" onMouseUp="$P().restart_server('+idx+')"><b>Перезапуск</b></span>',
				'<span class="link" onMouseUp="$P().shutdown_server('+idx+')"><b>Выключить</b></span>'
			];
			if (server.disabled) actions = [];
			if (!server.master) {
				actions.push( '<span class="link" onMouseUp="$P().remove_server('+idx+')"><b>Удалить</b></span>' );
			}
			
			var group_names = [];
			var eligible = false;
			for (var idx = 0, len = app.server_groups.length; idx < len; idx++) {
				var group = app.server_groups[idx];
				var regexp = new RegExp( group.regexp, "i" );
				if (server.hostname.match(regexp)) {
					group_names.push( group.title );
					if (group.master) eligible = true;
				}
			}
			
			var jobs = find_objects( app.activeJobs, { hostname: server.hostname } );
			var num_jobs = jobs.length;
			
			var cpu = 0;
			var mem = 0;
			if (server.data && server.data.cpu) cpu += server.data.cpu;
			if (server.data && server.data.mem) mem += server.data.mem;
			for (idx = 0, len = jobs.length; idx < len; idx++) {
				var job = jobs[idx];
				if (job.cpu && job.cpu.current) cpu += job.cpu.current;
				if (job.mem && job.mem.current) mem += job.mem.current;
			}
			
			var tds = [
				'<div class="td_big">' + self.getNiceGroup(null, server.hostname, col_width) + '</div>',
				(server.ip || 'н/д').replace(/^\:\:ffff\:(\d+\.\d+\.\d+\.\d+)$/, '$1'),
				group_names.length ? group_names.join(', ') : '(Ничего)',
				server.master ? '<span class="color_label green"><i class="fa fa-check">&nbsp;</i>Главный</span>' : (eligible ? '<span class="color_label purple">Резервный</span>' : '<span class="color_label blue">Рабочий</span>'),
				num_jobs ? commify( num_jobs ) : '(Ничего)',
				get_text_from_seconds( server.uptime, true, true ).replace(/\bday\b/, 'дней'),
				short_float(cpu) + '%',
				get_text_from_bytes(mem),
				actions.join(' | ')
			];
			
			if (server.disabled) tds.className = 'disabled';
			
			return tds;
		} );
		
		html += '<div style="height:25px;"></div>';
		html += '<center><table><tr>';
			html += '<td><div class="button" style="width:130px;" onMouseUp="$P().add_server()"><i class="fa fa-plus-circle">&nbsp;&nbsp;</i>Добавить...</div></td>';
		html += '</tr></table></center>';
		
		html += '<div style="height:30px;"></div>';
		
		// Server Groups
		
		var col_width = Math.floor( ((size.width * 0.9) + 300) / 6 );
		
		var cols = ['Заголовок', 'Совпадение имени хоста', '# кол-во серверов', '# кол-во событий', 'Класс', 'Действия'];
		
		html += '<div class="subtitle">';
			html += 'Группы серверов';
			// html += '<div class="clear"></div>';
		html += '</div>';
		
		// sort by title ascending
		this.server_groups = app.server_groups.sort( function(a, b) {
			// return (b.title < a.title) ? 1 : -1;
			return a.title.toLowerCase().localeCompare( b.title.toLowerCase() );
		} );
		
		// render table
		var self = this;
		html += this.getBasicTable( this.server_groups, cols, 'групп', function(group, idx) {
			var actions = [
				'<span class="link" onMouseUp="$P().edit_group('+idx+')"><b>Редактировать</b></span>',
				'<span class="link" onMouseUp="$P().delete_group('+idx+')"><b>Удалить</b></span>'
			];
			
			var regexp = new RegExp( group.regexp, "i" );
			var num_servers = 0;
			for (var hostname in app.servers) {
				if (hostname.match(regexp)) num_servers++;
			}
			
			var group_events = find_objects( app.schedule, { target: group.id } );
			var num_events = group_events.length;
			
			return [
				'<div class="td_big" style="white-space:nowrap;"><span class="link" onMouseUp="$P().edit_group('+idx+')">' + self.getNiceGroup(group, null, col_width) + '</span></div>',
				'<div class="ellip" style="font-family:monospace; max-width:'+col_width+'px;">/' + encode_entities(group.regexp) + '/</div>',
				// group.description || '(Без описания)',
				num_servers ? commify( num_servers) : '(Ничего)',
				num_events ? commify( num_events ) : '(Ничего)',
				group.master ? '<b>Главный подходящий</b>' : 'Только рабочий',
				actions.join(' | ')
			];
		} );
		
		html += '<div style="height:25px;"></div>';
		html += '<center><table><tr>';
			html += '<td><div class="button" style="width:130px;" onMouseUp="$P().edit_group(-1)"><i class="fa fa-plus-circle">&nbsp;&nbsp;</i>Добавить...</div></td>';
		html += '</tr></table></center>';
		
		html += '</div>'; // padding
		html += '</div>'; // sidebar tabs
		
		this.div.html( html );
	},
	
	add_server_from_list: function(idx) {
		// add a server right away, from the nearby list
		var server = this.servers[idx];
		
		app.showProgress( 1.0, "Добавляем сервер..." );
		app.api.post( 'app/add_server', { hostname: server.ip || server.hostname }, function(resp) {
			app.hideProgress();
			app.showMessage('success', "Сервер успешно добавлен.");
			// self['gosub_servers'](self.args);
		} ); // api.post
	},
	
	add_server: function() {
		// show dialog allowing user to enter an arbitrary hostname to add
		var html = '';
		
		// html += '<div style="font-size:12px; color:#777; margin-bottom:15px;">Обычно серверы должны автоматически добавлять себя в кластер, если они находятся в пределах диапазона широковещательной рассылки UDP (т. е. находятся в одной локальной сети).  Добавлять сервер вручную нужно только в особых случаях, например. если он удаленно размещен в другом центре обработки данных или сети.</div>';
		
		// html += '<div style="font-size:12px; color:#777; margin-bottom:20px;">Обратите внимание, что новый сервер не может уже быть главным сервером или частью другого кластера серверов '+app.name+', и текущий главный сервер должен иметь возможность доступа к нему.</div>';
		
		html += '<center><table>' + 
			// get_form_table_spacer() + 
			get_form_table_row('Имя хоста или IP:', '<input type="text" id="fe_as_hostname" style="width:280px" value="" spellcheck="false"/>') + 
			get_form_table_caption("Введите имя хоста или IP-адрес сервера, который вы хотите добавить.") + 
		'</table></center>';
		
		app.confirm( '<i class="mdi mdi-desktop-tower mdi-lg">&nbsp;&nbsp;</i>Добавить сервер', html, "Добавить", function(result) {
			app.clearError();
			
			if (result) {
				var hostname = $('#fe_as_hostname').val().toLowerCase();
				if (!hostname) return app.badField('fe_as_hostname', "Введите имя хоста или IP-адрес сервера.");
				if (!hostname.match(/^[\w\-\.]+$/)) return app.badField('fe_as_hostname', "Введите действительное имя хоста или IP-адрес сервера.");
				if (app.servers[hostname]) return app.badField('fe_as_hostname', "Этот сервер уже находится в кластере.");
				Dialog.hide();
				
				app.showProgress( 1.0, "Добавляем сервер..." );
				app.api.post( 'app/add_server', { hostname: hostname }, function(resp) {
					app.hideProgress();
					app.showMessage('success', "Сервер успешно добавлен.");
					// self['gosub_servers'](self.args);
				} ); // api.post
			} // user clicked add
		} ); // app.confirm
		
		setTimeout( function() { 
			$('#fe_as_hostname').focus();
		}, 1 );
	},
	
	remove_server: function(idx) {
		// remove manual server after user confirmation
		var server = this.servers[idx];
		
		var jobs = find_objects( app.activeJobs, { hostname: server.hostname } );
		if (jobs.length) return app.doError("К сожалению, вы не можете удалить сервер, на котором выполняются активные задания.");
		
		// proceed with remove
		var self = this;
		app.confirm( '<span style="color:red">Удалить сервер</span>', "Вы уверены, что хотите удалить сервер <b>"+server.hostname+"</b>?", "Удалить", function(result) {
			if (result) {
				app.showProgress( 1.0, "Удаление сервера..." );
				app.api.post( 'app/remove_server', server, function(resp) {
					app.hideProgress();
					app.showMessage('success', "Сервер успешно удален.");
					// self.gosub_servers(self.args);
				} );
			}
		} );
	},
	
	edit_group: function(idx) {
		// edit group (-1 == new group)
		var self = this;
		var group = (idx > -1) ? this.server_groups[idx] : {
			title: "",
			regexp: "",
			master: 0
		};
		var edit = (idx > -1) ? true : false;
		var html = '';
		
		html += '<table>';
		
		// Internal ID
		if (edit && this.isAdmin()) {
			html += get_form_table_row( 'ID группы', '<div style="font-size:14px;">' + group.id + '</div>' );
			html += get_form_table_caption( "Внутренний ID группы, используемый для вызовов API. Это невозможно изменить." );
			html += get_form_table_spacer();
		}
		
		html += 
			get_form_table_row('Название группы:', '<input type="text" id="fe_eg_title" size="25" value="'+escape_text_field_value(group.title)+'"/>') + 
			get_form_table_caption("Введите название группы серверов, короткое и понятное.") + 
			get_form_table_spacer() + 
			get_form_table_row('Соответствие имени хоста:', '<input type="text" id="fe_eg_regexp" size="30" style="font-family:monospace; font-size:13px;" value="'+escape_text_field_value(group.regexp)+'" spellcheck="false"/>') + 
			get_form_table_caption("Введите регулярное выражение для автоматического назначения серверов в эту группу по их именам хостов, например \"^mtx\\d+\\.\".") + 
			get_form_table_spacer() + 
			get_form_table_row('Класс сервера:', '<select id="fe_eg_master">' + render_menu_options([ [1,'Главный подходящий'], [0,'Только рабочий'] ], group.master, false) + '</select>') + 
			get_form_table_caption("Выберите, могут ли серверы в группе стать основным сервером или работать только как рабочие.") + 
		'</table>';
		
		app.confirm( '<i class="mdi mdi-server-network">&nbsp;&nbsp;</i>' + (edit ? "Редактировать группу серверов" : "Добавить группу серверов"), html, edit ? "Сохранить" : "Добавить", function(result) {
			app.clearError();
			
			if (result) {
				group.title = $('#fe_eg_title').val();
				if (!group.title) return app.badField('fe_eg_title', "Введите название группы серверов.");
				group.regexp = $('#fe_eg_regexp').val().replace(/^\/(.+)\/$/, '$1');
				if (!group.regexp) return app.badField('fe_eg_regexp', "Введите регулярное выражение для группы серверов.");
				
				try { new RegExp(group.regexp); }
				catch(err) {
					return app.badField('fe_eg_regexp', "Недопустимое регулярное выражение: " + err);
				}
				
				group.master = parseInt( $('#fe_eg_master').val() );
				Dialog.hide();
				
				// pro-tip: embed id in title as bracketed prefix
				if (!edit && group.title.match(/^\[(\w+)\]\s*(.+)$/)) {
					group.id = RegExp.$1;
					group.title = RegExp.$2;
				}
				
				app.showProgress( 1.0, edit ? "Сохранение группы..." : "Добавляем группу..." );
				app.api.post( edit ? 'app/update_server_group' : 'app/create_server_group', group, function(resp) {
					app.hideProgress();
					app.showMessage('success', "Группа серверов " + (edit ? "сохранена" : "добавлена") + " успешно.");
					// self['gosub_servers'](self.args);
				} ); // api.post
			} // user clicked add
		} ); // app.confirm
		
		setTimeout( function() { 
			if (!$('#fe_eg_title').val()) $('#fe_eg_title').focus();
		}, 1 );
	},
	
	delete_group: function(idx) {
		// delete selected server group
		var group = this.server_groups[idx];
		
		// make sure user isn't deleting final master group
		if (group.master) {
			var num_masters = 0;
			for (var idx = 0, len = this.server_groups.length; idx < len; idx++) {
				if (this.server_groups[idx].master) num_masters++;
			}
			if (num_masters == 1) {
				return app.doError("К сожалению, вы не можете удалить последнюю группу основных подходящих серверов.");
			}
		}
		
		// check for events first
		var group_events = find_objects( app.schedule, { target: group.id } );
		var num_events = group_events.length;
		if (num_events) return app.doError("К сожалению, вы не можете удалить группу, которой назначены события.");
		
		// proceed with delete
		var self = this;
		app.confirm( '<span style="color:red">Удалить группу серверов</span>', "Вы уверены, что хотите удалить группу серверов <b>"+group.title+"</b>? Отменить это действие невозможно.", "Удалить", function(result) {
			if (result) {
				app.showProgress( 1.0, "Удаление группы..." );
				app.api.post( 'app/delete_server_group', group, function(resp) {
					app.hideProgress();
					app.showMessage('success', "Группа серверов успешно удалена.");
					// self.gosub_servers(self.args);
				} );
			}
		} );
	},
	
	restart_server: function(idx) {
		// restart server after confirmation
		var self = this;
		var server = this.servers[idx];
		
		app.confirm( '<span style="color:red">Перезапустить сервер</span>', "Вы уверены, что хотите перезагрузить сервер <b>"+server.hostname+"</b>? Все задания сервера будут прерваны.", "Перезапустить", function(result) {
			if (result) {
				app.showProgress( 1.0, "Перезапуск сервера..." );
				app.api.post( 'app/restart_server', server, function(resp) {
					app.hideProgress();
					app.showMessage('success', "Сервер перезапускается в фоновом режиме.");
					// self.gosub_servers(self.args);
				} );
			}
		} );
	},
	
	shutdown_server: function(idx) {
		// shutdown server after confirmation
		var self = this;
		var server = this.servers[idx];
		
		app.confirm( '<span style="color:red">Выключить сервер</span>', "Вы уверены, что хотите выключить сервер <b>"+server.hostname+"</b>? Все задания сервера будут прерваны.", "Выключить", function(result) {
			if (result) {
				app.showProgress( 1.0, "Выключение сервера..." );
				app.api.post( 'app/shutdown_server', server, function(resp) {
					app.hideProgress();
					app.showMessage('success', "Сервер отключается в фоновом режиме.");
					// self.gosub_servers(self.args);
				} );
			}
		} );
	}
	
});